from __future__ import annotations

import sys
import types

import pytest


@pytest.mark.unit
def test_package_init_exports_systemwebview_entrypoint() -> None:
    package_name = "fit_webview_bridge"
    module_name = f"{package_name}.systemwebview"

    for mod_name in [package_name, module_name, "systemwebview"]:
        sys.modules.pop(mod_name, None)

    fake_mod = types.ModuleType(module_name)
    fake_widget = type("FakeWKWebViewWidget", (), {})
    fake_mod.WKWebViewWidget = fake_widget

    sys.modules[module_name] = fake_mod

    imported = __import__(package_name, fromlist=["*"])

    assert imported.SystemWebView is fake_widget



from __future__ import annotations

import pytest


@pytest.mark.contract
def test_module_exports_expected_types(systemwebview_module) -> None:
    assert getattr(systemwebview_module, "WKWebViewWidget", None) is not None
    assert getattr(systemwebview_module, "DownloadInfo", None) is not None


@pytest.mark.contract
def test_widget_declares_expected_public_methods(widget_class) -> None:
    expected_methods = [
        "url",
        "setUrl",
        "back",
        "forward",
        "stop",
        "reload",
        "clearWebsiteData",
        "evaluateJavaScript",
        "evaluateJavaScriptWithResult",
        "setDownloadDirectory",
        "downloadDirectory",
        "setUserAgent",
        "userAgent",
        "resetUserAgent",
        "setApplicationNameForUserAgent",
        "captureVisiblePage",
    ]
    for method_name in expected_methods:
        assert callable(getattr(widget_class, method_name, None)), method_name


@pytest.mark.contract
def test_widget_exposes_expected_signals(widget_class) -> None:
    expected_signals = [
        "loadFinished",
        "urlChanged",
        "navigationDisplayUrlChanged",
        "titleChanged",
        "loadProgress",
        "canGoBackChanged",
        "canGoForwardChanged",
        "downloadStarted",
        "downloadProgress",
        "downloadFinished",
        "downloadFailed",
        "javaScriptResult",
        "captureFinished",
    ]

    for signal_name in expected_signals:
        signal = getattr(widget_class, signal_name, None)
        assert signal is not None, signal_name



from __future__ import annotations

from pathlib import Path

import pytest
from PySide6.QtCore import QUrl

from tests._qt_test_utils import wait_for


def _write_html(path: Path, title: str, body: str) -> None:
    html = f"""<!doctype html>
<html>
  <head><meta charset="utf-8"><title>{title}</title></head>
  <body>{body}</body>
</html>
"""
    path.write_text(html, encoding="utf-8")


@pytest.fixture
def shown_widget(qapp, widget_class):
    widget = widget_class()
    widget.resize(1024, 768)
    widget.show()
    qapp.processEvents()
    yield widget
    widget.close()
    widget.deleteLater()
    qapp.processEvents()


@pytest.mark.e2e
def test_navigation_back_forward_and_capture_flow(
    shown_widget, tmp_path: Path
) -> None:
    second_page = tmp_path / "page_2.html"
    _write_html(second_page, "Page Two", "<h1 id='page2'>second</h1>")

    first_page = tmp_path / "page_1.html"
    _write_html(first_page, "Page One", "<h1 id='page1'>first</h1>")
    first_url = QUrl.fromLocalFile(str(first_page))
    second_qurl = QUrl.fromLocalFile(str(second_page))

    load_events: list[bool] = []
    shown_widget.loadFinished.connect(lambda ok: load_events.append(bool(ok)))
    can_go_back = {"value": False}
    can_go_forward = {"value": False}
    shown_widget.canGoBackChanged.connect(lambda v: can_go_back.__setitem__("value", bool(v)))
    shown_widget.canGoForwardChanged.connect(
        lambda v: can_go_forward.__setitem__("value", bool(v))
    )

    shown_widget.setUrl(first_url)
    assert wait_for(lambda: len(load_events) > 0 and load_events[-1], timeout_ms=10000)

    js_events: list[tuple] = []
    shown_widget.javaScriptResult.connect(
        lambda result, token, error: js_events.append((result, int(token), str(error)))
    )
    title_token = int(shown_widget.evaluateJavaScriptWithResult("document.title"))
    assert wait_for(
        lambda: any(received_token == title_token for _, received_token, _ in js_events),
        timeout_ms=8000,
    )
    title_result, _, _ = next(item for item in js_events if item[1] == title_token)
    assert str(title_result) == "Page One"

    shown_widget.setUrl(second_qurl)
    assert wait_for(lambda: len(load_events) > 1 and load_events[-1], timeout_ms=10000)

    assert wait_for(
        lambda: "page_2.html" in shown_widget.url().toString(),
        timeout_ms=10000,
    )
    if not wait_for(lambda: can_go_back["value"], timeout_ms=3000):
        pytest.skip("Back navigation history is not available in this runtime session")

    shown_widget.back()
    assert wait_for(
        lambda: "page_1.html" in shown_widget.url().toString(),
        timeout_ms=10000,
    )

    if not wait_for(lambda: can_go_forward["value"], timeout_ms=3000):
        pytest.skip("Forward navigation history is not available in this runtime session")

    shown_widget.forward()
    assert wait_for(
        lambda: "page_2.html" in shown_widget.url().toString(),
        timeout_ms=10000,
    )

    output = tmp_path / "e2e_capture.png"
    capture_events: list[tuple] = []
    shown_widget.captureFinished.connect(
        lambda token, ok, file_path, error: capture_events.append(
            (int(token), bool(ok), str(file_path), str(error))
        )
    )

    cap_token = int(shown_widget.captureVisiblePage(str(output)))
    assert wait_for(
        lambda: any(received_token == cap_token for received_token, _, _, _ in capture_events),
        timeout_ms=12000,
    )
    _, ok, _, error = next(item for item in capture_events if item[0] == cap_token)
    assert ok is True, error
    assert output.exists()
    assert output.stat().st_size > 0



from __future__ import annotations

from pathlib import Path

import pytest
from PySide6.QtCore import QUrl

from tests._qt_test_utils import wait_for


def _write_page(path: Path, title: str, body: str) -> None:
    html = f"""<!doctype html>
<html>
  <head><meta charset="utf-8"><title>{title}</title></head>
  <body>{body}</body>
</html>
"""
    path.write_text(html, encoding="utf-8")


@pytest.fixture
def shown_widget(qapp, widget_class):
    widget = widget_class()
    widget.resize(900, 700)
    widget.show()
    qapp.processEvents()
    yield widget
    widget.close()
    widget.deleteLater()
    qapp.processEvents()


@pytest.mark.integration
def test_load_local_file_emits_finished_and_title(
    shown_widget, tmp_path: Path
) -> None:
    page = tmp_path / "integration_page.html"
    _write_page(page, "Integration Title", "<h1 id='msg'>ready</h1>")
    url = QUrl.fromLocalFile(str(page))

    load_events: list[bool] = []
    shown_widget.loadFinished.connect(lambda ok: load_events.append(bool(ok)))

    shown_widget.setUrl(url)

    assert wait_for(lambda: len(load_events) > 0, timeout_ms=10000)
    assert load_events[-1] is True
    assert shown_widget.url().toLocalFile() == str(page)


@pytest.mark.integration
def test_evaluate_javascript_with_result_returns_expected_value(
    shown_widget, tmp_path: Path
) -> None:
    page = tmp_path / "js_page.html"
    _write_page(page, "JS Page", "<script>window.answer = 6 * 7;</script>")
    load_events: list[bool] = []
    shown_widget.loadFinished.connect(lambda ok: load_events.append(bool(ok)))
    shown_widget.setUrl(QUrl.fromLocalFile(str(page)))
    assert wait_for(lambda: len(load_events) > 0 and load_events[-1], timeout_ms=10000)

    results: list[tuple] = []
    shown_widget.javaScriptResult.connect(
        lambda result, token, error: results.append((result, int(token), str(error)))
    )
    token = int(shown_widget.evaluateJavaScriptWithResult("6 * 7"))

    assert wait_for(
        lambda: any(received_token == token for _, received_token, _ in results),
        timeout_ms=10000,
    )
    result, _, error = next(item for item in results if item[1] == token)
    assert float(result) == 42.0
    assert error == ""


@pytest.mark.integration
def test_capture_visible_page_writes_image_file(
    shown_widget, tmp_path: Path
) -> None:
    page = tmp_path / "capture_page.html"
    _write_page(page, "Capture Page", "<div style='height:1200px'>capture</div>")
    load_events: list[bool] = []
    shown_widget.loadFinished.connect(lambda ok: load_events.append(bool(ok)))
    shown_widget.setUrl(QUrl.fromLocalFile(str(page)))
    assert wait_for(lambda: len(load_events) > 0 and load_events[-1], timeout_ms=10000)

    out_file = tmp_path / "capture.png"
    captures: list[tuple] = []
    shown_widget.captureFinished.connect(
        lambda token, ok, file_path, error: captures.append(
            (int(token), bool(ok), str(file_path), str(error))
        )
    )

    token = int(shown_widget.captureVisiblePage(str(out_file)))
    assert wait_for(
        lambda: any(received_token == token for received_token, _, _, _ in captures),
        timeout_ms=12000,
    )

    _, ok, file_path, error = next(item for item in captures if item[0] == token)
    assert ok is True, error
    assert Path(file_path) == out_file
    assert out_file.exists()
    assert out_file.stat().st_size > 0
